@@ -5,9 +5,11 @@
 package gzip
 
 import (
+	"bufio"
 	"bytes"
 	oldgz "compress/gzip"
 	"crypto/rand"
+	"errors"
 	"io"
 	"os"
 	"strings"
@@ -407,11 +409,14 @@ func TestDecompressor(t *testing.T) {
 }
 
 func TestIssue6550(t *testing.T) {
-	f, err := os.Open("testdata/issue6550.gz")
+	b, err := os.ReadFile("testdata/issue6550.gz")
 	if err != nil {
 		t.Fatal(err)
 	}
-	gzip, err := NewReader(f)
+	// issue6550.gz has invalid flag bit set.
+	// TestReservedBits tests that. Fix it up for this test.
+	b[3] &= 0x1f
+	gzip, err := NewReader(bytes.NewReader(b))
 	if err != nil {
 		t.Fatalf("NewReader(testdata/issue6550.gz): %v", err)
 	}
@@ -434,6 +439,21 @@ func TestIssue6550(t *testing.T) {
 	}
 }
 
+func TestReservedBits(t *testing.T) {
+	f, err := os.Open("testdata/issue6550.gz")
+	if err != nil {
+		t.Fatal(err)
+	}
+	defer f.Close()
+	_, err = NewReader(f)
+	if err == nil {
+		t.Fatalf("Reserved bits did not result in error")
+	}
+	if !errors.Is(err, ErrHeader) {
+		t.Fatal("Expected ErrHeader, got", err)
+	}
+}
+
 func TestInitialReset(t *testing.T) {
 	var r Reader
 	if err := r.Reset(bytes.NewReader(gunzipTests[1].gzip)); err != nil {
@@ -537,6 +557,7 @@ func TestWriteTo(t *testing.T) {
 		t.Fatal(err)
 	}
 	wtbuf := &bytes.Buffer{}
+
 	written, err := dec.WriteTo(wtbuf)
 	if err != nil {
 		t.Fatal(err)
@@ -708,3 +729,43 @@ func TestTruncatedGunzip(t *testing.T) {
 		}
 	}
 }
+
+func TestBufferedPartialCopyGzip(t *testing.T) {
+	var (
+		in           = []byte("hello\nworld")
+		compressedIn []byte
+	)
+
+	var buf bytes.Buffer
+	gzw := NewWriter(&buf)
+	if _, err := gzw.Write(in); err != nil {
+		panic(err)
+	}
+	if err := gzw.Flush(); err != nil {
+		panic(err)
+	}
+	if err := gzw.Close(); err != nil {
+		panic(err)
+	}
+
+	compressedIn = buf.Bytes()
+
+	gz, err := NewReader(bytes.NewReader(compressedIn))
+	if err != nil {
+		t.Errorf("constructing a reader: %v", err)
+	}
+
+	br := bufio.NewReader(gz)
+	if _, err := br.ReadBytes('\n'); err != nil {
+		t.Errorf("reading to the first newline: %v", err)
+	}
+
+	var out bytes.Buffer
+	_, err = io.Copy(&out, br)
+	if !bytes.Equal(out.Bytes(), []byte("world")) {
+		t.Errorf("unexpected output when reading the remainder")
+	}
+	if err != nil {
+		t.Errorf("reading the remainder: %v", err)
+	}
+}
